
#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Java.
# @File         : gen_pb_model.py
# @Time         : 2020-03-13 18:03
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : IntelliJ IDEA
# @Description  :


# from wandb import magic
import numpy as np
from tensorflow.keras.layers import *
from tensorflow.keras.models import Model
from tensorflow.keras import backend as K
from tensorflow.keras import optimizers
import tensorflow
import tensorflow as tf
from tensorflow import keras

print(tf.__version__)
# import tensorflow.keras.backend as K

from sklearn.datasets import make_classification

X, y = make_classification(100000, 16)

input_shape = X.shape[1:]
model_input = Input(input_shape)
x = Dense(128)(model_input)
x = Dense(64)(x)
model_out = Dense(1, activation='sigmoid')(x)

model = Model(inputs=model_input, outputs=model_out)
model.compile(
    loss='binary_crossentropy',
    optimizer='adam',
    metrics=['acc', tf.keras.metrics.AUC()])

model.fit(X, y, batch_size=128, epochs=1)
model.save('keras.h5')
# saved_model_cli show --dir tfmodel --all

model.save('tfmodel', save_format="tf")
# tf.saved_model.save(model, "./pb")

if __name__ == '__main__':
    import os

    os.system("saved_model_cli show --dir tfmodel --all")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : simbert
# @Time         : 2020/11/20 2:50 下午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  :

import tensorflow as tf

from meutils.pipe import *
from meutils.bert_utils import SimBert
from meutils.np_utils import normalize
from meutils.io_utils import tf_read_text
from meutils.date_utils import date_difference

# Bert
simbert = SimBert()
_ = simbert.texts2vec(['第一条文本', '第二条文本'])
logger.info(_)

# Path
hdfs = "/fds/infra-client/bin/hdfs --cluster zjyprc-hadoop dfs"
in_path = "/user/h_data_platform/platform/browser/browser_intermediate/data/o2o_title_add_daily"
out_path = "/user/h_data_platform/platform/browser/browser_intermediate/data/o2o_simbert_vector"

    # 加载数据

for days in range(1, 13):

    logger.info("加载数据")
    date = date_difference('%Y%m%d', days=days)
    file = f'/fds/data/{date}_data.tsv'

    data = tf_read_text(tf, f"{in_path}/date={date}/*")


    def parse_fn(data):
        for line in tqdm(data):
            line = line.numpy().decode()
            line = line.strip().split('\t')
            if len(line) == 7:  # 7个字段
                yield line
            else:
                logger.warning(line)


    data = list(parse_fn(data))

    cloumns = data[0]
    df = (
        pd.DataFrame(data[1:], columns=cloumns)[lambda df: df.iloc[:, 0] != cloumns[0]]  # 多文件去掉表头
            .drop_duplicates("title", ignore_index=True)
            .assign(title=lambda df: df['title'].astype(str))
    )

    # 转换向量
    logger.info("转换向量")

    df['vector'] = normalize(simbert.texts2vec(tqdm(df['title']), is_lite=0)[:, range(0, 768, 3)]).tolist()
    df.to_csv(file, '\t', index=False)





# # 上传数据: ls -a && rm .data.tsv.crc
# logger.info("上传数据")
# os.system(f"rm .*.crc && touch _SUCCESS")
# os.system("kinit -k -t /fds/data/h_browser.keytab h_browser@XIAOMI.HADOOP && klist")
#
# os.system(f"{hdfs} -mkdir -p {out_path}/date=000_{date}")  # todo: 覆盖
# os.system(f"{hdfs} -put -f {file} {out_path}/date=000_{date}")
# os.system(f"{hdfs} -put -f _SUCCESS {out_path}/date=000_{date}")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : Redis
# @Time         : 2020/6/18 7:21 下午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

# from redis import StrictRedis, ConnectionPool
#
#
# class Redis(object):
#
#     def __init__(self, host: str, port=None):
#         if host.__contains__(":"):
#             host, port = host.split(":")
#
#         # 多个Redis实例共享一个连接池，避免每次建立、释放连接的开销。
#         pool = ConnectionPool(host=host, port=port, decode_responses=True)
#         self.client = StrictRedis(connection_pool=pool)
#
#     def randomkey(self):
#         return self.client.randomkey()


from rediscluster import RedisCluster


class Redis(object):

    def __init__(self, host_ports=None, is_dev=True):
        if host_ports is not None:
            pass
        elif is_dev:
            host_ports = "10.38.164.94:7000,10.38.164.94:7001,10.38.164.94:7002,10.38.164.94:7003,10.38.164.94:7004,10.38.164.94:7005"

        self.startup_nodes = [dict(zip(['host', 'port'], host_port.split(':'))) for host_port in host_ports.split(',')]

        self.client = RedisCluster(startup_nodes=self.startup_nodes.copy(), decode_responses=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : articleinfo
# @Time         : 2020/12/9 2:52 下午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.zk_utils import get_zk_config
from mi.db import Mongo

mongo_cache_url = get_zk_config()['mongo_cache_url']

mongo = Mongo()
ac = mongo.db.articleinfo


def get_articleinfo(docid='0003899b202871b7fd3dab15f2f9549a'):
    """
    todo: 定义ac结构
    :param docid:
    :return:
    """
    doc = ac.find_one({'id': docid})
    if doc:
        articleinfo = eval(doc['ac'])

    else:
        url = f"{mongo_cache_url}/ac?id={docid}"
        r = requests.request('get',
                             url)  # 包含插入逻辑 doc = {'id': docid, 'ac': str(articleinfo)} _ = ac.insert(doc, continue_on_error=False)
        articleinfo = r.json()['result']
    return articleinfo



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : nlp_utils
# @Time         : 2020/10/20 4:16 下午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


from LAC import LAC
from functools import lru_cache

lac_lac = LAC(mode='lac')
lac_seg = LAC(mode='seg')


@lru_cache(512)
def nlp_cut(text, mode='seg', sep=""):
    lac = lac_seg if mode == 'seg' else lac_lac
    _ = lac.run(text)
    return sep.join(_) if sep else _



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : o2o_bert_vector
# @Time         : 2020/11/11 1:19 下午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

from tql.pipe import *

os.environ['LOG_PATH'] = f'/home/work/yuanjie/{Path(__file__).name}.log'

from tql.utils.logger_utils import logger
from tql.utils.np_utils import normalize
from tql.nlp.utils.bert_utils import Bert
from bert4keras.snippets import sequence_padding

# Bert
bert = Bert('/home/work/yuanjie/chinese_simbert_L-12_H-768_A-12')

tokenizer = bert.tokenizer
simbert_encoder = bert.simbert_encoder


def texts2vec(texts, is_lite=1, batch_size=1000, maxlen=64):
    X = []
    S = []
    for text in texts:
        token_ids, segment_ids = tokenizer.encode(text, maxlen=maxlen)
        X.append(token_ids)
        S.append(segment_ids)

    data = [sequence_padding(X, length=maxlen), sequence_padding(S, length=maxlen)]

    del text, X, S
    gc.collect()

    vecs = simbert_encoder.predict(data, batch_size=batch_size)

    if is_lite:
        vecs = vecs[:, range(0, 768, 3)]

    return normalize(vecs)


def convert_vector_from_file(file='data.tsv'):
    df = pd.read_csv(file, '\t').drop_duplicates("title", ignore_index=True)
    df['vector'] = texts2vec(tqdm(df['title'].astype(str))).tolist()
    df.to_csv(file, '\t', index=False)


# Path
file = 'data.tsv'
hdfs = "~/infra-client/bin/hdfs --cluster zjyprc-hadoop dfs"
in_path = "/user/h_data_platform/platform/browser/browser_intermediate/data/o2o_title_add_daily"
out_path = "/user/h_data_platform/platform/browser/browser_intermediate/data/o2o_simbert_vector"

# 下载数据
date = str(get_date(-1))[:10].replace('-', '')
logger.info("下载数据")
os.system(f"{hdfs} -getmerge {in_path}/date={date} {file}")

# 转换向量
logger.info("转换向量")
convert_vector_from_file(file)

# 上传数据: ls -a && rm .data.tsv.crc
logger.info("上传数据")
os.system(f"rm .*.crc && touch _SUCCESS")
os.system(f"{hdfs} -mkdir -p {out_path}/date={date}")  # todo: 覆盖
os.system(f"{hdfs} -put -f {file} {out_path}/date={date}")
os.system(f"{hdfs} -put -f _SUCCESS {out_path}/date={date}")

# # 删除数据
# logger.info("删除数据")
# os.system(f"rm *.crc")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : s
# @Time         : 2019-11-01 10:23
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : https://docs.streamlit.io/api.html?highlight=markdown#display-progress-and-status
# https://docs.streamlit.io/cli.html
# streamlit run your_script.py --server.port 80
# 8051
# https://github.com/streamlit/streamlit/blob/develop/examples/file_uploader.py

import os
import time
import datetime
import requests
import streamlit as st
import pandas as pd
from mi.db import Mongo, ANN
from bson import json_util

pd.set_option('display.max_rows', 100)
pd.set_option('display.max_columns', 100)
pd.set_option('max_colwidth', 128)

# db
mongo = Mongo()
ann = ANN()

# side
st.sidebar.markdown('**SideBar**')
biz = st.sidebar.selectbox('业务场景', ('xindao_ann', 'other'), index=0)

mongo_collection = mongo.db.__getitem__(f"{biz}")

## mongo
st.sidebar.markdown('---')
st.sidebar.markdown('**Mongo Find**')
filter_ = st.sidebar.text_input('filter', {'xindaoid': 44517})  # **kwargs
_ = mongo_collection.find_one(eval(filter_))
st.sidebar.json(json_util.dumps(_))

# body
st.title("检索相似文章")
text = st.text_input('text', value="火锅香不香")
topk = st.text_input('topk', value="10")
category = st.text_input('category', value="美食")

query_embedding = (
    requests.get(f"http://tql.algo.browser.miui.srv/bert/simbert?texts=['{text}']")
        .json()['vectors'][0]
)

query_hybrid = {
    "bool": {
        "must": [
            {
                "vector": {
                    "embedding": {"topk": int(topk), "query": [query_embedding], "metric_type": "IP",
                                  "params": {"nprobe": 1}}
                }
            }
        ]
    }
}

if category:
    category_ = mongo_collection.find_one({'category': category})['category_']
    query_hybrid['bool']['must'].append({'term': {"category_": [category_]}})

entities = ann.client.search("demo", query_hybrid)[0]
id2score = dict(zip(entities.ids, entities.distances))

docs = mongo_collection.find({"xindaoid": {'$in': entities.ids}})
df = pd.DataFrame(list(docs)).drop(['_id', 'category_', 'vector'], 1)
df['distance'] = df['xindaoid'].map(id2score)

st.dataframe(df.sort_values("distance", ascending=False))






#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : mi_run
# @Time         : 2020/11/10 6:12 下午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

"""
'console_scripts': [
    'mi-run=mi.mi_run:cli',
]
"""
import os
import fire
from pathlib import Path
from mi.utils import get_module_path


class Run(object):
    """doc"""

    def __init__(self, **kwargs):
        pass

    def tasks_list(self, tasks='tasks'):
        return list(Path(get_module_path(f'../{tasks}', __file__)).glob('*'))

    def task(self, app_file='demo.py', nohup=0, python='python'):
        """mi-run - task xindao_ann"""
        if '/' not in app_file:
            app_file = list(Path(get_module_path('../tasks', __file__)).glob(f'*{app_file}*'))[0]
        cmd = f"{python} {app_file}"
        self._run_cmd(cmd, nohup)

    def _run_cmd(self, cmd, nohup=0):
        cmd = f"nohup {cmd} &" if nohup else cmd
        return os.system(cmd)


def cli():
    fire.Fire(Run)


if __name__ == '__main__':
    print(Run().tasks_list())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : get_article
# @Time         : 2020/12/8 4:29 下午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.zk_utils import get_zk_config
from meutils.date_utils import date_difference
from mi.db.Hive import Hive
from mi.db import Mongo

mongo_cache_url = get_zk_config()['mongo_cache_url']

hive = Hive('/home/work/keytab/h_browser.keytab')
mongo = Mongo()
ac = mongo.db.articleinfo

date = date_difference('%Y%m%d', days=1)
sql = f"""
select data as id, date
from browser.date_biz_partition
where biz='nh_test' and date={date} 
"""

df = pd.read_sql(sql, hive.conn)


def request(docid='0003899b202871b7fd3dab15f2f9549a', method='get'):
    url = f"{mongo_cache_url}/ac?id={docid}"
    r = requests.request('get', url)
    r.encoding = r.apparent_encoding
    return r.json()['result']


def get_and_insert_ac(docid='yidian_0R18y0XA'):
    # 多请求
    # ac.replace_one({'id': 'yidian_0R18y0XA'}, {'$set': {'id': 'id'}}) # 替换原始值{**d1, **d2}
    if ac.find_one({'id': docid}):
        logger.info(f'dup key: {docid}')
    else:
        doc = {'id': docid, 'ac': str(request(docid))}
        _ = ac.insert(doc, continue_on_error=False)


old_num = ac.count_documents({})
tqdm(df.id) | xThreadPoolExecutor(get_and_insert_ac, 20)

logger.info(f"新增内容{ac.count_documents({}) - old_num}条")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : nh_choice_ann
# @Time         : 2020/12/10 3:58 下午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


from meutils.pipe import *
from meutils.hash_utils import murmurhash
from meutils.zk_utils import zk_cfg, get_zk_config
from mi.db import Mongo, ANN, Hive
from milvus import DataType

mongo = Mongo()
nh_choice = mongo.db.nh_choice
mongo_ids = set(nh_choice.distinct('id'))

ann = ANN('10.119.28.8')
collection_name = 'nh_choice'


# # Create collection
# from milvus import DataType
#
# collection_param = {
#     "fields": [
#         #  Milvus doesn't support string type now, but we are considering supporting it soon.
#         #  {"name": "title", "type": DataType.STRING},
#         {"name": "category", "type": DataType.INT32},  # murmurhash()
#         {"name": "source", "type": DataType.INT32},
#         {"name": "title_vec", "type": DataType.FLOAT_VECTOR, "params": {"dim": 768}},
#     ],
#     "segment_row_limit": 4096,
#     #         "auto_id": False
# }
#
# ann.create_collection(collection_name, collection_param)
# ann.create_index(collection_name, 'title_vec')

# Insert entities
def get_articleinfo(docid):
    return requests.get(f"{zk_cfg.ac_assistant_url}/{docid}").json().get('item', {})


def get_title_vec(title):
    return requests.get(f"{zk_cfg.simbert_url}{title}").json().get('vectors')[0]


docids = set(get_zk_config('/mipush/nh_label/1').split()) - mongo_ids

if len(docids):
    logger.info(f"获取ArticleInfo")
    acs = docids | xThreadPoolExecutor(get_articleinfo, 16) | xlist
    df = (
        pd.DataFrame(acs)[['id', 'title', 'category', 'source']]
            .drop_duplicates('title')
            .assign(category=lambda df: df.category.str[0])
            .assign(category_hash=lambda df: df.category.map(murmurhash))
            .assign(source_hash=lambda df: df.source.map(murmurhash))
    )

    logger.info(f"获取TitleVec")
    df['title_vec'] = df.title.tolist() | xThreadPoolExecutor(get_title_vec, 8) | xlist

    # entities = ann.client.get_collection_info(collection_name)['fields']
    # for entity in entities:
    #     entity['values'] = df[entity['name'] + "_hash"]

    entities = [
        {"name": "title_vec", "type": DataType.FLOAT_VECTOR, "values": df['title_vec'].tolist()},
        {"name": "category", "type": DataType.INT32, "values": df['category_hash'].tolist()},
        {"name": "source", "type": DataType.INT32, "values": df['source_hash'].tolist()},
    ]

    df['ann_id'] = ann.client.insert(collection_name, entities)

    nh_choice.insert_many(df.to_dict('r'), False)

    logger.info(f"新增优质内容「{len(df)}」条")
else:
    logger.info(f"新增优质内容「{len(df)}」条")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : nni_app
# @Time         : 2020/9/3 1:24 下午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

import os
import time
import pandas as pd
from typing import Optional
from fastapi import FastAPI, Form, Depends, File, UploadFile, Body
from pydantic import BaseModel
from starlette.staticfiles import StaticFiles
from starlette.requests import Request
from starlette.responses import \
    RedirectResponse, FileResponse, HTMLResponse, PlainTextResponse
from starlette.status import *
import numpy as np

# experiment

ROUTE = "/nni"
app = FastAPI(
    debug=True,
    openapi_url=f"{ROUTE}/openapi.json",
    docs_url=f"{ROUTE}/docs",
    redoc_url=f"{ROUTE}/redoc",
    swagger_ui_oauth2_redirect_url=f"{ROUTE}/docs/oauth2-redirect"
)


@app.get("/nni/download")
def hdfs():
    cmd = f"""
    /usr/bin/kinit -k -t /h_browser.keytab h_browser@XIAOMI.HADOOP
    rm -rf ~/nni/experiments/*
    /infra-client/bin/hdfs --cluster zjyprc-hadoop dfs -get /user/h_data_platform/platform/browser/logs/data/nni/* ~/nni/experiments
    nnictl create --config config.yml
    """
    os.system(cmd)
    files = os.popen('ls ~/nni/experiments').read().strip().split()
    return {'files': files}


@app.get("/nni/experiment/{experiment_id}")
def read_root(experiment_id):
    cmd = f"""
    
    nnictl stop --all
    sleep 3
    nnictl resume {experiment_id}
    """
    os.system(cmd)

    return RedirectResponse("0.0.0.0:8080")


if __name__ == '__main__':
    import socket
    import uvicorn

    me = socket.gethostname() == 'yuanjie-Mac.local'
    main_file = __file__.split('/')[-1].split('.')[0]

    # app参数必须是个字符串才能开启热更新
    uvicorn.run(app=f"{main_file}:app", host='0.0.0.0', port=8000, workers=1, reload=True, debug=True)
